include("queue.lua");

ZONE_HOUSE = 400;
GLOBAL_COMMAND_ID_SEED = 1;

ENTITY_TYPE_TELEPORTER = 1;
ENTITY_TYPE_VENDOR = 2;
ENTITY_TYPE_PETHUNTER = 4;
ENTITY_TYPE_AUCTIONHOUSE = 8;
ENTITY_TYPE_BANK = 16;
ENTITY_TYPE_HOUSEMAID = 32;
ENTITY_TYPE_MAILBOX = 64;

MINIGAME_COT = "COT";
MINIGAME_SURVIVAL = "Survival";
MINIGAME_GG = "GG";
MINIGAME_AT = "AT";

function ControllerInstance()
	if (__controller == nil) then __controller = CController(); end;
	return __controller;
end;

CController = class(
	function(self)
		self.DEBUG = true;
		self.config = {};
		
		--Daily Quest to run
		self.config.daily = { enabled = false};
		self.config.daily.Id = 424180;
		self.config.daily.StarterId = 118072;
		self.config.daily.StarterZone = 14;
		self.config.daily.EnderId = 118072;
		self.config.daily.EnderZone = 14;
		
		--Housemaid types
		local housemaids = { enabled = true};
		local maid = {id=113803, run="1,5", type="cooking", name="Mila Black"};
		table.insert(housemaids, maid);
		
		maid = {id=113776, run="1,3", type="battle", name="Istolil Hill"};
		table.insert(housemaids, maid);
		
		maid = {id=113799, run="1,6", type="crafting", name="Clark Zorlan"};
		table.insert(housemaids, maid);
		
		self.config.housemaids = housemaids;
		self.config.housemaidsLastDone = nil;
		
		--Minigames
		local minigames = { enabled = false};
		minigames.COT = { id=MINIGAME_COT, last = nil, starter=111813, starterGUID=nil, startZone=2, wpls={"cot_tele"}};
		minigames.Survival = { id=MINIGAME_SURVIVAL, last = nil, starter=111813,  starterGUID=nil, startZone=2, wpls={"survivalR5"}};
		minigames.GG = { id=MINIGAME_GG, last = nil, starter=110694,  starterGUID=nil, startZone=4, wpls={"Malatinas_Goblins_Wp"}};
		minigames.AT = { id=MINIGAME_AT, last = nil, starter=110538,  starterGUID=364, startZone=2, wpls={"VaranasToATStart", "AT", "ATToOS"}};
		self.config.minigames = minigames;
		--ControllerInstance().config.minigames.GG.last = os.time();
		--ControllerInstance().config.housemaids.enabled = false;
		--ControllerInstance().config.minigames.enabled = false;
		--State
		self.actionList = CQueue();
		self.currentAction = nil;

		--EVENTS
		self.events = {};
		self.events.onComplete = function(_state)
			if (_state == WPT_EVENT_FINISHED) then
				--remove from action list --> need a queue here
				self.currentAction = nil;
				self:Think();
			end;
		end;
		
		self.think = {};
		self.think.lastCallTime = os.time();
	end
);

function CController:print(_text)
	if (self.DEBUG) then
		printf(_text.."\n");
	end;
end;



-- ***********************	Action management *************************
-- In theory, this method should be called at the end of each action to
-- to move the controller on to the next action. It may also get called
-- if the bot is stuck, or something else occurs - so this needs to be 
-- handled somehow.
function CController:Think()
	self:print("==============\nTHINK\n==============");
	self:print(tostring(self.think.lastCallTime));
	local time = os.time();
	local elapsed = time - self.think.lastCallTime;
	self.think.lastCallTime = time;
	
	if (self.currentAction) then
		self:print("currentAction: "..tostring(self.currentAction.name));
	else
		self:print("currentAction: None");
	end;
	self:print("actionList.Empty: "..tostring(self.actionList:isEmpty()));
	if (self.currentAction) then
		--check for errored state (stuck etc)
		self:print("  Perfoming an action - do nothing new");
		return;
	end;
	
	--we aint doing nuffink!
	--if we currently have a plan of action, then do that
	if (not self.actionList:isEmpty()) then
		self:print("  Moving on to next action");
		--are we still performing the current action? or are we ready to move on to the next action;
		self:SetAction(self.actionList:pop());
		return;
	end;
	
	-- =================== Decide what to do ====================
	
	self:print("Checking minigames");
	-- ============ minigames
	if (self.config.minigames.enabled) then
		for k, v in pairs(self.config.minigames) do
			if (k ~= "enabled") then
				if (not self:hasDoneMinigame(v)) then
					if (self:doMinigame(v)) then
						--self:SetAction(self.actionList:pop());
						return;
					end;
				end;
			end;
		end;
	end;
	
	self:print("Checking daily's");
	-- ============ daily's
	if (self.config.daily.enabled) then
		if (not self:HasCompletedDailies()) then
			self:print("  Aint done dailies");
			local wpl = self:buildPathToNPC(self.config.daily.StarterZone, self.config.daily.StarterId);
			local dailyWPL = CWaypointList();
			local paths = self:getQuestFile(self.config.daily.Id);
			if (#paths > 0) then
				local questFile = getExecutionPath().."/waypoints/quests/"..tostring(self.config.daily.Id)..".xml";
				dailyWPL:load(questFile, true);
				dailyWPL.notifyOfEvent = self.events.onComplete;
				
				--load the daily script
				if (wpl and dailyWPL) then 
					self.actionList:push(self:buildAction("Run to Dailies", wpl));
					self.actionList:push(self:buildAction("Do Dailies", dailyWPL));
					self:print("ActionList.Count: "..self.actionList:count());
				end;
				return;
			else
				self:print("Could not find waypoint file for daily quest "..tostring(self.config.daily.Id));
			end;
		end;
	end;

	-- ============ Housemaids
	self:print("Checking Housemaids");
	if (self.config.housemaids.enabled) then
		--do we need to do housemaids?
		--have we done them at all yet? or has it been 2 hours?
		--need to also check for potions in backpack
		if ((not self.config.housemaidsLastDone) or ((60*60*2) > os.time() - self.config.housemaidsLastDone)) then
			self:print("  Going to housemaids");
			local wpl = self:buildPathToHouse();
			self:print("Have path to house");
			
			--load the daily script
			if (wpl) then 
				self:print("Have path to house xml AND execute xml");
				self.actionList:push(self:buildAction("Goto house", wpl));
				self.actionList:push(self:buildAction("Talk to housekeepers", "ControllerInstance():runHousemaids()", "code"));
				self.actionList:push(self:buildAction("Exit house", "player:target_NPC(\"Housekeeper\");\nChoiceOptionByName(\"leave\");\nwaitForLoadingScreen(30);\n", "code"));
				self:print("ActionList.Count: "..self.actionList:count());
			end;
			self:SetAction(self.actionList:pop());
			return;
		end;
	end;
	
	--what next!!!!?? questing / resting / KS farming???
	self:print("  Finished");
	
end;

function CController:saveState()
	filename = getExecutionPath() .. "/logs/controllerState.txt";
	file, err = io.open(filename, "w");
	if( not file ) then
		self:print("Could not write to state file");
	end

	for k,v in pairs(self.config.minigames) do
		if (v.last ~= nil) then
			file:write(self.id.."\t"..self.last);
		end;
	end;
	
	file:write(text.."\n")
	file:close();
end;

function CController:hasDoneMinigame(_game)
	local dt = nil;
	
	dt = _game.last;
	
	if (dt == nil) then return false; end;
	local cmp = os.time();
	cmp = cmp-(24*60*60);
	
	self:print("hadDoneMinigame(".._game.id.."): dt="..tostring(dt)..", cmp="..tostring(cmp));
	return (dt >= cmp);
end;

function CController:doMinigame(_minigame)
	--get path to starter
	local wplMove = self:buildPathToNPC(_minigame.startZone, _minigame.starter, _minigame.starterGUID);
	local wpls = {};
	local wplsName = {};
	for k,v in pairs(_minigame.wpls) do
		local wpl = CWaypointList();
		wpl:load(getExecutionPath().."/waypoints/"..v..".xml", true);
		wpl.Filename = v;
		wpl.notifyOfEvent = self.events.onComplete;	
		table.insert(wpls, wpl);
	end;
	
	if (wplMove and (#wpls > 0)) then
		self.actionList:push(self:buildAction("Move to minigame starter", wplMove));
		self.actionList:push(self:buildAction("Target starter", "player:target_NPC("..tostring(_minigame.starter).."); yrest(100);", "code"));
		for k,v in pairs(wpls) do
			self.actionList:push(self:buildAction("Running minigame wpl: "..tostring(_minigame.id), v));
		end;
		--add an update to set minigame.last to that date
		self.actionList:push(self:buildAction("Updating last run time for '"..tostring(_minigame.id).."'", "ControllerInstance().config.minigames."..tostring(_minigame.id)..".last = os.time();", "code"));
		self:print("ActionList.Count: "..self.actionList:count());
		return true;
	end;
	return false;
end;

function CController:getQuestFile(_questId)
	self:print("Getting quest file for quest "..tostring(_questId));
	local questDir = getExecutionPath().."/waypoints/quests";
	self:print("  - quest dir "..tostring(questDir));
	local files = getDirectory(questDir);
	local count = 0;
	local pathlist = { }
	
	for i,v in pairs(files) do
		self:print("  - testing against "..tostring(v));
		if( string.find (v,tostring(_questId)..".*xml",1) ) then
			self:print("    - found "..tostring(v));
			count = count + 1;
			pathlist[count] = { };
			pathlist[count].folder = _folder;
			pathlist[count].filename = v;
		end
	end;
	
	return pathlist;
end;
			

function CController:buildAction(_name, _wpl, _type)
	_type = _type or "wpl";
	return {name=_name, WPL=_wpl, type=_type};
end

function CController:Clear()
	self.currentAction = nil;
	while (not self.actionList:isEmpty()) do self.actionList:pop(); end;
end;

function CController:SetAction(_action)
	self:print("New action specified: "..tostring(_action.name));
	self.currentAction = _action;
	if (_action) then
		while (_action and (_action.type ~= "wpl")) do
			local fn = loadstring(_action.WPL);
			fn();
			
			_action = nil;
			
			if (not self.actionList:isEmpty()) then
				self:print("  Was a code action, moving on to next action");
				_action = self.actionList:pop();
				if (_action) then
					self:print("    new action specified: "..tostring(_action.name));
				end;
			end;
			self.currentAction = _action;
		end;

		if (_action and (_action.type == "wpl")) then
			__WPL = self.currentAction.WPL;
			self:print("  performing Load");
			__WPL:performLoad();
			self:print("  Load done");
		else
			endMoveNow = true;
			loadPaths("wander", rp_to_load);
			__WPL:setRadius(settings.profile.options.WANDER_RADIUS);
			__WPL:setMode("wander");
			releaseKeys();
			SetCommsState("on");
			sendChatMessage("NavPoint", "end");
			endMoveNow = false;
			self:Think();
		end;
	else
		endMoveNow = true;
		loadPaths("wander", rp_to_load);
		__WPL:setRadius(settings.profile.options.WANDER_RADIUS);
		__WPL:setMode("wander");
		releaseKeys();
		SetCommsState("on");
		sendChatMessage("NavPoint", "end");
		endMoveNow = false;
		self:Think();
	end;
	
	return;
end;


function CController:HasCompletedDailies()
	local dailyQuestCount, dailyQuestsPerDay = RoMScript("Daily_count()");
	self:print(tostring(dailyQuestCount).." of "..tostring(dailyQuestsPerDay).." daily quests completed");
	return (dailyQuestCount == dailyQuestsPerDay);
end;

function CController:HasHousemaidPots()
	local result = true;
	if (countGoodie) then
		if (1 > countGoodie("luck")) then return false; end;
		if (1 > countGoodie("casting")) then return false; end;
		if (1 > countGoodie("pdmg")) then return false; end;
	end;
	return result;
end;

function CController:runHousemaids()
	--if housemaids enabled and we haven't done them for 2 hours
	if ((self.config.housemaids.enabled) and ((os.time()-self.config.housemaidsLastDone) > (60*60*2))) then
		for k,v in pairs(self.config.housemaids) do
			if (k ~= "enabled") then
				self:runHousemaid(v);
			end;
		end;
		
		self.config.housemaidsLastDone = os.time();
	end;
end;

function CController:getHousemaidIndex(_housemaid)
	local count = RoMScript("Houses_GetServantCount()");
	for i=1,count,1 do
		local DBID, name, sex, character, month, day, horoscope, race, 
		Affinity, AffinityMax, Charisma, CharismaMax, 
		Fatigue, FatigueMax, Magic, MagicMax, Battle, BattleMax, Defense, DefenseMax, Cooking, CookingMax, Inventive, InventiveMax = RoMScript("Houses_GetServantInfo("..tostring(i)..")");		
		self:print("Comparing "..tostring(_housemaid.name).." to dbid: "..tostring(DBID).." name: "..tostring(name).." char: "..tostring(character));
		if (name == _housemaid.name) then return i; end;
	end;
	
	return -1;
end;

function CController:runHousemaid(_housemaid)
	if (_housemaid) then
		self:print("running housemaid "..tostring(_housemaid.id));
		if (player:target_NPC(_housemaid.id)) then
			local housemaidIndex = self:getHousemaidIndex(_housemaid);
			self:print("index = "..tostring(housemaidIndex));
			
			if (housemaidIndex > -1) then
			
			   local function LevelOne(skill)
				  player:target_NPC(_housemaid.id);
				  self:print("Leveling skill "..skill)
				  ChoiceOptionByName(skill);
				  yrest(200);
			   end			
			
				local done = false;
				repeat
					local DBID, name, sex, character, month, day, horoscope, race, 
					Affinity, AffinityMax, Charisma, CharismaMax, 
					Fatigue, FatigueMax, Magic, MagicMax, Battle, BattleMax, Defense, DefenseMax, Cooking, CookingMax, Inventive, InventiveMax = RoMScript("Houses_GetServantInfo("..tostring(housemaidIndex)..")");	

					if (Fatigue > 85) then done = true; end;
					if (Fatigue > 75) then
						if (Affinity < AffinityMax) then
							self:print("Fatigue > 75, can only level affinity");
							LevelOne("chat");
						else
							self:print("Fatigue > 75, and Affinity is at max. Done.");
							done = true;
						end;
					end;
					
					if ((Inventive < Affinity) and (Inventive < InventiveMax)) then
						LevelOne("Crafting")
					elseif Affinity < 49 then
						LevelOne("Chat")
					else
						LevelOne(_housemaid.type);
					end;
				 
				until done;
			end;
		end;
	end;
end

function CController:getNearest(_entityType)
	self:print("finding path to "..tostring(_entityType));
	local request = sprintf("MAP\1FINDNEAREST\1ENTITYTYPE\1%d\2%d\2%d\2%d\2%d\0", player.X, player.Y, player.Z, getZoneId(), _entityType);
	self:print("sending: "..tostring(request));
	local link = CServerLink();
	local data = link:send(request, true);
	link:close();
	self:print("received: "..tostring(data));
	
	local dataArray = SplitString(data, "\1"); -- returns Name, Id, uniqueId, Zone, Path
	local npcName = dataArray[1];
	local npcId = dataArray[2];
	local npcUniqueId = dataArray[3];
	if (npcUniqueId=="") then npcUniqueId = nil; end;
	local zone = dataArray[4];
	local path = dataArray[5];
	self:print("pathData: "..tostring(path));
	local pathXML = self:inflatePath(path, 2);
	self:print("inflated XML: "..tostring(pathXML));
	
	return npcName, npcId, npcUniqueId, zone, pathXML;
end;

function CController:buildPathToHouse()
	--self:print("buildPathToHouse");
	local npcName, npcId, npcUniqueId, zone, pathXML = self:getNearest(ENTITY_TYPE_HOUSEMAID);
	self:print("XML to house: "..tostring(pathXML));
	local result = CWaypointList();
	result:loadFromString(pathXML);
	result.notifyOfEvent = self.events.onComplete;
	
	local wpEnter = result:getLastWaypoint();
	local   action = "local z=getZoneId();\n";
	action = action.."local t=os.time();\n";
	action = action.."while ((z ~= "..tostring(ZONE_HOUSE)..") and (60 > (os.time()-t))) do\n"; --while not in house zone, or it has taken more than 1 minute
	action = action.."  player:target_NPC("..tostring(npcId)..");\n";
	action = action.."  ChoiceOptionByName(\"enter\");\n";
	action = action.."  waitForLoadingScreen(30);\n";
	action = action.."  z=getZoneId();\n";
	action = action.."end;\n";
	action = action.."yrest(1000);\n";
	wpEnter.Action = action;
	self:print("Added action: "..tostring(action));
	
	return result;
end;

function CController:buildExecuteWPL(_code)
	local sPath = "<?xml version=\"1.0\" encoding=\"utf-8\"?><waypoints>\n";
	--code to create a waypoint at current character pos
	_code = _code.."\nlocal wp = CWaypoint(player.X, player.Y, player.Z); __WPL.Waypoints = { wp };\n"; 
	sPath = sPath.."<onLoad>setWaypointLooping(false);\n".._code.."\n__WPL:performFinished();\n</onLoad>\n";
	--sPath = sPath.."<!-- #   0 --><waypoint x=\""..tostring(player.X).."\" z=\""..tostring(player.Z).."\" y=\""..tostring(player.Y).."\"></waypoint>\n";
	sPath = sPath.."</waypoints>";
	self:print("buildExecuteWPL: made: \n"..sPath);
	local result = CWaypointList();
	result:loadFromString(sPath);
	
	result.notifyOfEvent = self.events.onComplete;
	return result;
end;

function CController:inflatePath(_path, _delimiter, _onLoad, _onUnload)
	local _delim = string.char(_delimiter);
	local sPath = "<?xml version=\"1.0\" encoding=\"utf-8\"?><waypoints>\n";
	--self:print("sPath1: "..tostring(sPath));
	
	if (_onLoad) then
		sPath = sPath.."<onLoad>".._onLoad.."\n</onLoad>\n";
	end;
	if (_onUnload) then
		sPath = sPath.."<onUnload>".._onUnload.."\n</onUnload>\n";
	end;
	
	if (string.len(_path) > 0) then
		local points = SplitString(_path, _delim);
		local index = 1;
		local data = "";
		for k,v in ipairs(points) do
			--self:print("Working on: "..tostring(v));
			if (string.len(v) > 0) then
				data=SplitString(v, string.char(_delimiter+1));
				sPath = sPath.."<!-- #   "..tostring(index).." --><waypoint x=\""..tostring(data[1]).."\" z=\""..tostring(data[3]).."\" y=\""..tostring(data[2]).."\">"..tostring(data[4]).."</waypoint>\n";
				index = index+1;
			end;
		end;
		sPath = sPath.."<!-- #   "..tostring(index).." --><waypoint x=\""..tostring(data[5]).."\" z=\""..tostring(data[7]).."\" y=\""..tostring(data[6]).."\"></waypoint>\n";
	else
		--handle case where we are already in the right location
		sPath = sPath.."<!-- #   "..tostring(index).." --><waypoint x=\""..tostring(player.X).."\" z=\""..tostring(player.Z).."\" y=\""..tostring(player.Y).."\"></waypoint>\n";
	end;
	
	sPath = sPath.."</waypoints>";
	
	return sPath;
end;

function CController:buildPathToNPC(_zone, _npcid, _uniqueId)
	self:print("buildPathToNPC("..tostring(_zone)..", "..tostring(_npcid)..", "..tostring(_uniqueId)..")");
	local pathReq = "";
	if (_uniqueId) then 
		pathReq = sprintf("MAP\1GETROUTE\1%s\2%s\2%s\2%s\2%s\2%s\2%s\0", player.X, player.Y, player.Z, getZoneId(), _zone, _npcid, _uniqueId);
	else
		pathReq = sprintf("MAP\1GETROUTE\1%s\2%s\2%s\2%s\2%s\2%s\0", player.X, player.Y, player.Z, getZoneId(), _zone, _npcid);
	end;


	local link = CServerLink();
	local path = link:send(pathReq, true);
	--self:print("Path: "..tostring(path));
	link:close();
	
	local pathXML = self:inflatePath(path, 1, "setWaypointLooping(false);\n");
	--self:print("xml: \n"..pathXML);
	local result = CWaypointList();
	result:loadFromString(pathXML);
	result.notifyOfEvent = self.events.onComplete;
	
	return result;
end;
