CGameMonitors = class();

GAME_MONITOR_MIN_TIME = 1.4;

function CGameMonitors:constructor(limit)
	self.Q = CQueue();
	self.limit = limit;
	self.active = false;
	self.timeToCheckOneMessage = GAME_MONITOR_MIN_TIME; --this represents the time it takes to check a channel for a message
	
	self.events = {
					{["name"]='GM', ["urgent"]=true}, 
					{["name"]='SAY', ["urgent"]=false}, 
					{["name"]='WHISPER', ["urgent"]=true}, 
					{["name"]='GUILD', ["urgent"]=false}, 
					{["name"]='ZONE', ["urgent"]=false}, 
					{["name"]='YELL', ["urgent"]=false}, 
					{["name"]='PARTY', ["urgent"]=true}, 
					{["name"]='TRADE', ["urgent"]=false}, 
					{["name"]='SYSTEM', ["urgent"]=false},
					{["name"]='CHANNEL', ["urgent"]=false},
					};
end

function CGameMonitors:start()
	for k,v in pairs(self.events) do
		EventMonitorStart('all'..v.name, 'CHAT_MSG_'..v.name);
		printf("Starting monitor for: "..v.name.." urgent="..tostring(v.urgent).."\n");
	end;
	self.active = true;
end

function CGameMonitors:stop()
	self.active = false;
	for k,v in pairs(self.events) do
		EventMonitorStop('all'..v.name, 'CHAT_MSG_'..v.name);
		printf("Stopped monitor for: "..v.name.."\n");
	end;

end

function CGameMonitors:checkChannel(channel, mustEndBefore)
	if (not self.active) then return end;
	
--	printf("checking channel "..channel.."\n");
	local _time, more, msg, name = EventMonitorCheck("all"..channel)
	repeat
		if (_time ~= nil) then 
			printf("Inserting: "..channel.." - "..msg.."\n");
			self.Q:push({ ["time"]=_time, ["channel"]=channel, ["message"]=msg, ["name"]=name });
			self.count = self.count + 1;
			if (self.count >= self.limit) then
				return;
			end
		end
		if (more) then 
			--if ((mustEndBefore) and ((os.clock()+self.timeToCheckOneMessage) > mustEndBefore)) then
			--	return;
			--end;
			_time, more, msg, name = EventMonitorCheck("all"..channel);
		end;
			
	until (not more)
	
	return;
end

function CGameMonitors:checkForMessages(mustEndBefore)
	if (not self.active) then return 0; end;

	self.count = 0;
	
	for k,v in pairs(self.events) do
		if ((mustEndBefore > -1) and ((os.clock()+self.timeToCheckOneMessage) > mustEndBefore)) then
			return self.count;
		end;
		
		self:checkChannel(v.name, mustEndBefore);
	end;	
	
--	printf("returning count "..tostring(self.count).."\n");
	return self.count;
end

function CGameMonitors:checkForUrgentMessages()
	if (not self.active) then return 0; end;

	self.count = 0;
	
	for k,v in pairs(self.events) do
		if (v.urgent == true) then
			self:checkChannel(v.name);
		end;
	end;	
	
--	printf("returning count "..tostring(self.count).."\n");
	return self.count;
end


function CGameMonitors:clear()
end

