--[[
	Manages different states. That's all. Nothing you haven't seen before.
]]
events = {}
include("state.lua");

StateManager = class();

function StateManager:constructor()
	printf("StateManager:constructor()\n");
	self.tick = 0;
	self.stateQueue = {};
	self.eventQueue = {};
	self.running = true; -- Set to false to stop the bot
end

-- Runs the update function for the current state.
function StateManager:run()
--	printf("StateMan:run(): "..self.tick.."\n");

	if( #self.stateQueue > 0 ) then
		self.stateQueue[#self.stateQueue]:update();
	end
end

-- Cycles through events and checks to see if anything meaningful happened.
function StateManager:handleEvents()
	for i,event in pairs(self.eventQueue) do
		local handled = false;
		-- See if the current state wants to handle this event
		if( #self.stateQueue > 0 ) then
			--printf("Calling "..tostring(self.stateQueue[#self.stateQueue].name).." for event "..tostring(event.event).."\n");
			handled = self.stateQueue[#self.stateQueue]:handleEvent(event.event, event.args);
		end

		-- If the state didn't handle it, then maybe the manager will.
		if( not handled ) then
			handled = self:handleEvent(event.event, event.args);
		end
	end

	-- Flush our queue.
	self:flushEvents();
end

-- Handle a single event. If we use it, return true. Else, false.
function StateManager:handleEvent(event, args)
	if( event == "Quit" ) then
		self.running = false;
		return true;
	end

	--printf("handling event: "..event.."\n");
	for k,events in pairs(events) do
		--printf("Name: "..events.name.." - count: "..#self.stateQueue.."\n");
		if( event == events.name and ((#self.stateQueue == 0) or (self.stateQueue[#self.stateQueue].name ~= event))) then
			if (settings.profile.options.DEBUG_STATES == true) then
				logger:log('info',"Switching to "..event);
				printf("Switching to "..event.."\n");
			end;
			
			local state = events.func(args);
			stateman:pushState(state);
			return true;
		end
	end
end

-- Returns the current running state (if there is one)
-- Otherwise, returns nil.
function StateManager:getState()
	if( #self.stateQueue > 0 ) then
		return self.stateQueue[#self.stateQueue];
	else
		return nil;
	end
end

-- Push a new state onto the queue. This will be the
-- state that gets run *next cycle*, not immediately.
function StateManager:pushState(newState)
	if( type(newState) == nil ) then
		error("Function expects a parameter.", 2);
	end

	if( type(newState) ~= "table" ) then
		error("Parameter is not a state object.", 2);
	end

	table.insert(self.stateQueue, newState);
end

-- Pops the top state off the queue.
function StateManager:popState(from)
	from = from or "unknown"
	if (settings.profile.options.DEBUG_STATES == true) then
		logger:log("info" ,"statemanager pop state from "..from.."\n")
		printf("statemanager pop state from "..from.."\n");
	end;
	
	if( #self.stateQueue > 0 ) then
		table.remove(self.stateQueue);
	end
end

-- Push a new event
function StateManager:pushEvent(event, from, args)
	from = from or ""
	logger:log("info" ,"statemanager push event "..event.." from "..from.."\n")
	table.insert(self.eventQueue, {["event"]=event, ["args"]=args});
end

-- Pop last event
function StateManager:popEvent(event)
	table.remove(self.eventQueue, #self.eventQueue);
end

-- Remove all events
function StateManager:flushEvents()
	self.eventQueue = {};
end

-- Remove all events
function StateManager:flushStates()
	self.stateQueue = {};
end
