--[[
	Child of State class.
]]

include("../state.lua");

p_wp_gtype = "";	-- global type for whole file: e.g. TRAVEL
p_wp_type = "";		-- type for normal waypoints
p_hp_type = "HARVEST";		-- type for harvest waypoints
p_harvest_command = "player:harvest();";
p_merchant_command = "player:merchant(\"%s\");";
p_targetNPC_command = "player:target_NPC(\"%s\");";
p_targetObj_command = "player:target_Object(\"%s\");";
p_choiceOption_command = "sendMacro(\"ChoiceOption(%d);\");";
p_mouseClickL_command = "player:mouseclickL(%d, %d, %d, %d);";

CreatePathState = class(State);

function CreatePathState:constructor(args)
	self.name = "CreatePath";
	self.wpKey 			= key.VK_NUMPAD1;	-- insert a movement point
	self.saveKey 		= key.VK_NUMPAD7;	-- save the waypoints
	self.resetKey	 	= key.VK_NUMPAD8;	-- restart waypoints script and discard changes
	self.popKey 		= key.VK_NUMPAD9
	self.keyWasDown = false;
	self.waypointSpread = args.spread or 100;
	
	self.path = {};
	self:AddWaypointHere();
	--self.lastWaypoint = CWaypoint(player.X, player.Z, player.Y);
	--self.path = {self.lastWaypoint};
end;

function CreatePathState:update()
	logger:log(LL_INFO, "CreatePathState:update()");

	local keyIsDown = false;
	local isInsertKey = false;
	local isSaveKey = false;
	local isResetKey = false;
	local isPopKey = false;

	isInsertKey = keyPressedLocal(self.wpKey);
	isSaveKey = keyPressedLocal(self.saveKey);
	isResetKey = keyPressedLocal(self.resetKey);
	isPopKey = keyPressedLocal(self.popKey);

	keyIsDown = isInsertKey or isSaveKey or isResetKey or isPopKey;
	
	if ((not self.keyWasDown) and keyIsDown) then
		printf("key was not down and now is\n");
		self.keyWasDown = true;
		self.wasKey = self.wpKey;
		if (isSaveKey) then
			self.wasKey = self.saveKey;
		elseif (isResetKey) then
			self.wasKey = self.resetKey;
		elseif (isPopKey) then
			self.wasKey = self.popKey;
		end;
		
		return;
	elseif (self.keyWasDown and (not keyIsDown)) then
		printf("key was down and no longer is\n");
		self.keyWasDown = false;
		isInsertKey = self.wasKey == self.wpKey;
		isSaveKey = self.wasKey == self.saveKey;
		isResetKey = self.wasKey == self.resetKey;
		isPopKey = self.wasKey == self.popKey;
		
		--user has released the key
		if (isInsertKey) then
			printf("Insert\n");
			self:AddWaypointHere();
			return;
		elseif (isResetKey) then
			printf("Reset\n");
			self.lastWaypoint = CWaypoint(player.X, player.Z, player.Y);
			self.path = {self.lastWaypoint};
			return;
		elseif (isSaveKey) then
			printf("Save\n");
			self:SaveWaypoints(self.path);
			return;
		elseif (isPopKey) then
			stateman:popState(self:toString());
			return;
		end;
	end;
	
	--otherwise we get here and should be saving waypoints!!!!
	local dist = distance(self.lastWaypoint.X, self.lastWaypoint.Z, self.lastWaypoint.Y, player.X, player.Z, player.Y);
	
	if (dist > self.waypointSpread) then
		self:AddWaypointHere();
	end;
end

function CreatePathState:SaveWaypoints(list)
	keyboardBufferClear();
	io.stdin:flush();
	--cprintf(cli.green, language[500]);	-- What do you want to name your path
	filename = sprintf("%s/waypoints/%s.xml", getExecutionPath(), os.date('%H%M'));
	printf("saving to: "..filename.."\n");

	file, err = io.open(filename, "w");
	if( not file ) then
		error(err, 0);
	end

	local openformat = "\t<!-- #%3d --><waypoint x=\"%d\" z=\"%d\" y=\"%d\"%s>%s";
	local closeformat = "</waypoint>\n";

	file:write("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
	local str = sprintf("<waypoints%s>\n", p_wp_gtype);	-- create first tag
	file:write(str);					-- write first tag

	local hf_line, tag_open, line_num = "", false, 1;
	for i,v in pairs(list) do
		if( v.wp_type == "HP" ) then -- Harvest point
			if( tag_open ) then hf_line = hf_line .. "\t" .. closeformat; end;
			hf_line = hf_line .. sprintf(openformat, line_num, v.X, v.Z, v.Y, p_hp_type, p_harvest_command) .. closeformat;
			line_num = line_num + 1
			tag_open = false;
		elseif( v.wp_type == "WP" ) then -- Waypoint
			if( tag_open ) then hf_line = hf_line .. "\t" .. closeformat; end;
			hf_line = hf_line .. sprintf(openformat, line_num, v.X, v.Z, v.Y, p_wp_type, "");
			line_num = line_num + 1
			tag_open = true;
		elseif( v.wp_type == "MER" ) then -- Merchant
			if( tag_open ) then
				hf_line = hf_line .. "\t\t" .. sprintf(p_merchant_command, string.gsub(v.npc_name, "\"", "\\\"")) .. "\n";
			else
				hf_line = hf_line .. sprintf(openformat, line_num, v.X, v.Z, v.Y, p_wp_type,
				"\n\t\t" .. sprintf(p_merchant_command, v.npc_name) ) .. "\n";
				line_num = line_num + 1
				tag_open = true;
			end
		elseif( v.wp_type == "NPC" ) then -- Open NPC Dialog
			if( tag_open ) then
				hf_line = hf_line .. "\t\t" .. sprintf(p_targetNPC_command, string.gsub(v.npc_name, "\"", "\\\"")) .. "\n";
			else
				hf_line = hf_line .. sprintf(openformat, line_num, v.X, v.Z, v.Y, p_wp_type,
				"\n\t\t" .. sprintf(p_targetNPC_command, v.npc_name) ) .. "\n";
				line_num = line_num + 1
				tag_open = true;
			end
		elseif( v.wp_type == "CO" ) then -- Choice Option
			if( tag_open ) then
				hf_line = hf_line .. "\t\t" .. sprintf(p_choiceOption_command, v.co_num) .. "\n";
			else
				hf_line = hf_line .. sprintf(openformat, line_num, v.X, v.Z, v.Y, p_wp_type,
				"\n\t\t" .. sprintf(p_choiceOption_command, v.co_num) ) .. "\n";
				line_num = line_num + 1
				tag_open = true;
			end
		elseif( v.wp_type == "MC" ) then -- Mouse click (left)
			if( tag_open ) then
				hf_line = hf_line .. "\t\t" .. sprintf(p_mouseClickL_command, v.mx, v.my, v.wide, v.high) .. "\n";
			else
				hf_line = hf_line .. sprintf(openformat, line_num, v.X, v.Z, v.Y, p_wp_type,
				"\n\t\t" .. sprintf(p_mouseClickL_command, v.mx, v.my, v.wide, v.high) ) .. "\n";
				line_num = line_num + 1
				tag_open = true;
			end
		elseif( v.wp_type == "COD" ) then -- Code
			if( tag_open ) then
				hf_line = hf_line .. "\t\t" .. v.com .. "\n";
			else
				hf_line = hf_line .. sprintf(openformat, line_num, v.X, v.Z, v.Y, p_wp_type,
				"\n\t\t" .. v.com ) .. "\n";
				line_num = line_num + 1
				tag_open = true;
			end
		elseif( v.wp_type == "OBJ" ) then -- Target Object
			if( tag_open ) then
				hf_line = hf_line .. "\t\t" .. sprintf(p_targetObj_command, v.obj_name) .. "\n";
			else
				hf_line = hf_line .. sprintf(openformat, line_num, v.X, v.Z, v.Y, p_wp_type,
				"\n\t\t" .. sprintf(p_targetObj_command, v.obj_name) ) .. "\n";
				line_num = line_num + 1
				tag_open = true;
			end
		end
	end

	-- If we left a tag open, close it.
	if( tag_open ) then
		hf_line = hf_line .. "\t" .. closeformat;
	end

	if( bot.ClientLanguage == "RU" ) then
		hf_line = oem2utf8_russian(hf_line);		-- language conversations for Russian Client
	end

	file:write(hf_line);
	file:write("</waypoints>");

--[[
	if( tag_open ) then
		file:write("\n\t</waypoint>\n</waypoints>\n");
	else
		file:write("</waypoints>\n");
	end
]]

	file:close();

end


function CreatePathState:AddWaypointHere()
	local wp = CWaypoint(player.X, player.Z, player.Y);
	wp.wp_type = "WP";
	self.lastWaypoint = wp;
	table.insert(self.path, wp);
	printf("Added waypoint #"..tostring(#self.path).."\n");
end;


function CreatePathState:handleEvent(event)
	if (event == "Idle") then
		return true;
	end;
	return false;
end

table.insert(events,{name = "CreatePath", func = CreatePathState})