--[[
	Encapsulates the logic for moving the player to a waypoint
]]

include("../state.lua");

FHS_LOCATE = 0;
FHS_FARM = FHS_LOCATE + 1;

FarmHousemaidsState = class(State);

function FarmHousemaidsState:constructor(args)
	self.name = "FarmHousemaidsState";
	args = args or {};
	
-- parameters
	self.allowTargetAquisition = args.allowTargetAquisition or false;
	self.oldAllowTargetting = __botState.allowTargetting;
	self.currentState = FHS_LOCATE;
	self.numberOfMaids = 0;
	self.maids = {};
end

function FarmHousemaidsState:update()
	logger:log(LL_DEBUG_STATES,self:toString()..":update()");

	__botState.allowTargetting = self.allowTargetAquisition;

	local function GetHousemaidInfo(index)
		local DBID, name, sex, character, month, day, horoscope, race, 
		Affinity, AffinityMax, Charisma, CharismaMax, 
		Fatigue, FatigueMax, Magic, MagicMax, Battle, BattleMax, Defense, DefenseMax, Cooking, CookingMax, Inventive, InventiveMax = RoMScript("Houses_GetServantInfo("..tostring(index)..")");
		
		return {["index"]=index, ["id"]=DBID, ["name"]=name, ["sex"]=sex, ["character"]=character, ["month"]=month, ["day"]=day, ["horoscope"]=horoscope, ["race"]=race, ["affinity"]=Affinity, ["maxAffinity"]=AffinityMax, ["charisma"]=Charisma, ["maxCharisma"]=CharismaMax, ["fatigue"]=Fatigue, ["maxFatigue"]=FatigueMax, ["magic"]=Magic, ["maxMagic"]=MagicMax, ["battle"]=Battle, ["maxBattle"]=BattleMax, ["defense"]=Defense, ["maxDefense"]=DefenseMax, ["cooking"]=Cooking, ["maxCooking"]=CookingMax, ["inventive"]=Inventive, ["maxInventive"]=InventiveMax};
	end;
	
	if (self.currentState == FHS_LOCATE) then
		self.numberOfMaids = RoMScript("Houses_GetServantCount()"); 
		self.maids = {};
		for i=1,self.numberOfMaids,1 do
			local maid = GetHousemaidInfo(i);
			table.insert(self.maids, maid);
		end;
		self.currentState = FHS_FARM;
		self.currentMaidIndex = 1;
	elseif (self.currentState == FHS_FARM) then
		--self.currentMaidIndex = self.currentMaidIndex + 1;
		if (self.currentMaidIndex <= self.numberOfMaids) then
			local maid = self.maids[self.currentMaidIndex];
			
			local function LevelOne(skill,t)
				local dist = distance(player.X, player.Z, player.Y, t.X, t.Z, t.Y);
				if (dist > 10) then
					stateman:pushEvent("Move", "FarmHousemaids", { allowTargetAquisition=self.allowTargetAquisition, ignoreCombatEvent=true, ["waypoint"]=t, range=10 });
					return;
				end;
				player:target_NPC(maid.name);
				printf("Leveling skill "..skill.."\n")
				ChoiceOptionByName(skill);
				self.maids[self.currentMaidIndex] = GetHousemaidInfo(maid.index); --update the housemaid info so we know the new values
			end

			printf("Running housemaid "..tostring(maid.name).."["..tostring(maid.index).."]\n");
			local t = player:findNearestNameOrId(maid.name);
			t = CPawn(t.Address);
			if (t == nil) then
				printf("Could not locate maid "..maid.name.."\n");
				self.currentMaidIndex = self.currentMaidIndex + 1;
				return;
			end;

			
			if (maid.fatigue > 85) then 
				self.currentMaidIndex = self.currentMaidIndex + 1;
				return;
			end;
			
			if (maid.fatigue > 75) then
				if (maid.affinity < maid.maxAffinity) then
					self:print("Fatigue > 75, can only level affinity");
					LevelOne("chat",t);
				else
					self:print("Fatigue > 75, and Affinity is at max. Done.");
					self.currentMaidIndex = self.currentMaidIndex + 1;
					return;
				end;
			end;
			
			if ((maid.inventive < maid.affinity) and (maid.inventive < maid.maxInventive)) then
				LevelOne("Crafting",t)
			elseif maid.affinity < 49 then
				LevelOne("Chat",t)
			else
				if (maid.cooking < maid.maxCooking) then
					LevelOne("Cooking",t);
				elseif (maid.magic < maid.maxMagic) then
					LevelOne("Magic",t);
				elseif (maid.battle < maid.maxBattle) then
					LevelOne("Battle",t);
				elseif (maid.defense < maid.maxDefense) then
					LevelOne("Protection",t);
				else
					self.currentMaidIndex = self.currentMaidIndex + 1;
				end;
			end;
			
		else
			stateman:popState(self:toString()..": complete");
			__botState.allowTargetting = self.oldAllowTargetting;
		end;
	end;
	



end

function FarmHousemaidsState:handleEvent(event)
	if (event == "Combat") and (self.ignoreCombatEvent == true) then
		return true;
	elseif (event == "Idle") then
		return true;
	end
end

function FarmHousemaidsState:clearState()
end;


table.insert(events,{name = "FarmHousemaids", func = FarmHousemaidsState})
