--[[
	Encapsulates the logic for moving the player to a waypoint
]]

include("../state.lua");

HS_SEARCHING = 0;
HS_MOVINGTO = HS_SEARCHING + 1;
HS_STARTINGHARVESTING = HS_MOVINGTO + 1;
HS_HARVESTING = HS_STARTINGHARVESTING +1;

HarvestState = class(State);

function HarvestState:constructor(args)
	self.name = "HarvestState";

-- parameters

	self.allowTargetAquisition = args.allowTargetAquisition or __botState.allowTargetting;
	self.oldAllowTargetting = __botState.allowTargetting
	self.ignoreCombatEvent = args.ignoreCombatEvent or false;
	self.target = args.target;
	self.lastHarvestedNodeAddr = nil;
	self.currentState = HS_SEARCHING;
	self.closestHarvestable = nil;

	self:clearState();
end

function HarvestState:nodeStillFound(node)
	local obj = nil;
	local objectList = __View:getObjectList();
	for i = 0,objectList:size() do
		obj = objectList:getObject(i);
		if( obj.Address == node.Address ) then
			return true;
		end
	end

	return false;
end;

function HarvestState:findNearestHarvestable(_id, ignore)
	player:updateXYZ()
	ignore = ignore or 0;
	local closest = nil;
	local obj = nil;
	local objectList = __View:getObjectList();
	for i = 0,objectList:size() do
		obj = objectList:getObject(i);

		if( obj ~= nil ) then
			if( obj.Type == PT_NODE and obj.Address ~= ignore and
				(_id == obj.Id or (not _id and database.nodes[obj.Id])) ) then
				local harvestThis = true;
				if( database.nodes[obj.Id] ) then
					local harvestType = database.nodes[obj.Id].Type;
					if( harvestType == NTYPE_WOOD and settings.profile.options.HARVEST_WOOD == false ) then
						harvestThis = false;
					elseif( harvestType == NTYPE_HERB and settings.profile.options.HARVEST_HERB == false ) then
						harvestThis = false;
					elseif( harvestType == NTYPE_ORE and settings.profile.options.HARVEST_ORE == false ) then
						harvestThis = false;
					end
				end

				if( harvestThis == true ) then
					local dist = distance(player.X, player.Z, obj.X, obj.Z);
					if( closest == nil ) then
						if( distance(player.X, player.Z, player.Y, obj.X, obj.Z, obj.Y) < settings.profile.options.HARVEST_DISTANCE ) then
							closest = obj;
						end
					else
						if( distance(player.X, player.Z, player.Y, obj.X, obj.Z, obj.Y) <
							distance(player.X, player.Z, player.Y, closest.X, closest.Z, closest.Y) ) then
							-- this node is closer
							closest = obj;
						end
					end
				end
			end
		end
	end

	return closest;
end

function HarvestState:update()
	logger:log(LL_DEBUG_STATES,self:toString()..":update()");
	__botState.allowTargetting = self.allowTargetAquisition;



	if ((self.currentState == HS_SEARCHING) or (self.currentState == HS_MOVINGTO)) then
		self.closestHarvestable = findNearestHarvestable(player.target, self.lastHarvestedNodeAddr);
		if( self.closestHarvestable == nil ) then
			printf(language[79]);
			return;
		end

		if( type(settings.profile.events.onHarvest) == "function" ) then
			arg1 = CPawn(self.closestHarvestable.Address);
			local status,result = pcall(settings.profile.events.onHarvest);
			if( status == false ) then
				local msg = sprintf("onHarvest error: %s", result);
				error(msg);
			end

			if( result == false ) then -- They chose to not harvest this in the event
				return;
			end
		end

		-- Check harvest skill level.
		if database.nodes[self.closestHarvestable.Id] then
			local harvestLevel = database.nodes[self.closestHarvestable.Id].Level
			local harvestType = database.nodes[self.closestHarvestable.Id].Type
			local craftLevel
			if harvestType == NTYPE_ORE then
				craftLevel = player:getCraftLevel(CRAFT_MINING)
			elseif harvestType == NTYPE_WOOD then
				craftLevel = player:getCraftLevel(CRAFT_WOODCUTTING)
			elseif harvestType == NTYPE_HERB then
				craftLevel = player:getCraftLevel(CRAFT_HERBALISM)
			end
			if harvestLevel > craftLevel then
				print(language[76]) -- Harvest skill level too low
				return false;
			end
		end

		cprintf(cli.yellow, language[95], self.closestHarvestable.Name);

		if( distance(player.X, player.Z, player.Y, self.closestHarvestable.X, self.closestHarvestable.Z, self.closestHarvestable.Y) > 80 ) then
			self.currentState = HS_MOVINGTO;
			stateman:pushEvent("Move", self:toString(), { ignoreCombatEvent=self.ignoreCombatEvent, ["waypoint"]=CPawn(self.closestHarvestable), range=10 });
			return;
		else
			self.currentState = HS_STARTINGHARVESTING;
		end
		
	end;
	
	if (self.currentState == HS_STARTINGHARVESTING) then
		if( nodeStillFound(self.closestHarvestable) ) then
			player:target(self.closestHarvestable.Address)
			yrest(100)
			Attack();
			yrest(100);
			player:updateHarvesting();
			if (player.Harvesting) then
				self.currentState = HS_HARVESTING;
			else
				self.currentState = HS_SEARCHING; --failed to start harvesting
			end;
			return;
		else
			self.currentState = HS_SEARCHING;
			return; -- Node disappeared for whatever reason...
		end

		if self.target and not database.nodes[self.closestHarvestable.Id] then -- The rest is not needed if not resource node
			stateman:popState(self.name.." completed");
			__botState.allowTargetting = self.oldAllowTargetting;
			return;
		end
	elseif (self.currentState == HS_HARVESTING) then
		if( deltaTime(getTime(), timeStart) > 45000 ) then
			-- Taking too long. Drop out.
			printf("Stop harvesting. Taking too long.\n");
			stateman:popState(self.name.." Taking too long");
			__botState.allowTargetting = self.oldAllowTargetting;
			return;
		end

	end;
end

function HarvestState:handleEvent(event)
	if (event == "Combat") and (self.ignoreCombatEvent == true) then
		return true;
	elseif (event=="Idle") then
		return true;
	end
end

function HarvestState:clearState()
end;


table.insert(events,{name = "Harvest", func = HarvestState})
