--[[
	Encapsulates the logic for moving the player to a waypoint
]]

include("../state.lua");

LS_START = 0;
LS_MOVINGTO = LS_START + 1;
LS_GETTING = LS_MOVINGTO + 1;

LootState = class(State);

function LootState:constructor(args)
	self.name = "LootState";

-- parameters
	self.currentState = LS_START;
	self.allowTargetAquisition = args.allowTargetAquisition or false;
	self.oldAllowTargetting = __botState.allowTargetting;
end

function LootState:update()
	logger:log(LL_DEBUG_STATES,self:toString()..":update()");

	if( settings.profile.options.LOOT ~= true and settings.profile.options.LOOT_SIGILS ~= true) then
		if( settings.profile.options.DEBUG_LOOT) then
			cprintf(cli.yellow, "[DEBUG] don't loot reason: settings.profile.options.LOOT ~= true and settings.profile.options.LOOT_SIGILS ~= true\n");
		end;
		return
	end

	if settings.profile.options.LOOT == true then 
		player:updateTargetPtr()
		if( player.TargetPtr == 0 ) then
			if( settings.profile.options.DEBUG_LOOT) then
				cprintf(cli.yellow, "[DEBUG] don't loot reason: player.TargetPtr == 0\n");
			end;
			stateman:popState(self:toString()..": don't loot reason: player.TargetPtr");
			__botState.allowTargetting = self.oldAllowTargetting;
			return
		end

		-- aggro and not loot in combat
		player:updateBattling()
		if( player.Battling  and settings.profile.options.LOOT_IN_COMBAT ~= true ) and
			player:findEnemy(true, nil, evalTargetDefault) then
			player:clearTarget()
			cprintf(cli.green, language[178]); 	-- Loot skiped because of aggro
			stateman:popState(self:toString()..": dont loot in combat.");
			__botState.allowTargetting = self.oldAllowTargetting;
			return
		end

		local target = CPawn.new(player.TargetPtr);

		if not target:exists() then
			if( settings.profile.options.DEBUG_LOOT) then
				cprintf(cli.yellow, "[DEBUG] don't loot reason: target == nil or target.Address == 0\n");
			end;
			stateman:popState(self:toString()..": don't loot reason: target nil or target address == 0");
			__botState.allowTargetting = self.oldAllowTargetting;
			return;
		end

		player:updateXYZ()
		target:updateXYZ()
		local dist = distance(player.X, player.Z, target.X, target.Z);
		local hf_x = player.X
		local hf_z = player.Z;
		local lootdist = 100;

		-- Set to combat distance; update later if loot distance is set
		if( settings.profile.options.COMBAT_TYPE == "ranged" ) then
			lootdist = settings.profile.options.COMBAT_DISTANCE;
		end

		if( settings.profile.options.LOOT_DISTANCE ) then
			lootdist = settings.profile.options.LOOT_DISTANCE;
		end

		if( dist > lootdist ) then 	-- only loot when close by
			self.currentState = LS_MOVINGTO;
			stateman:pushEvent("Move", "Loot", { ["allowTargetAquisition"]=false; ["waypoint"]=target;  ["range"] = lootdist; ["ignoreCombatEvent"]=true} );
			return;
		end


		local function looten()
			self.currentState = LS_GETTING;
			Attack()
			keyboardRelease( settings.hotkeys.MOVE_FORWARD.key);
			yrest(200)

		--	yrest(settings.profile.options.LOOT_TIME + dist*15); -- dist*15 = rough calculation of how long it takes to walk there
		--	inventory:updateSlotsByTime(settings.profile.options.LOOT_TIME + dist*15);
			local maxWaitTime = settings.profile.options.LOOT_TIME + dist*15 -- dist*15 = rough calculation of how long it takes to walk there
			local startWait = getTime()
			target:updateLootable()
			player:updateStance()
			while target.Lootable == true and player.Stance == 0 and deltaTime(getTime(), startWait) < maxWaitTime do
				player:updateActualSpeed()
				if player.ActualSpeed == 0 then Attack() yrest(100) end
				yrest(100)
				target:updateLootable()
				player:updateStance()
			end

			-- Wait for character to finish standing
			local starttime = os.clock()
			player:updateStance()
			while player.Stance ~= 0 and 2 > (os.clock() - starttime) do
				yrest(50)
				player:updateStance()
			end
		end

		target:updateLootable();
		local lootStart = os.clock()
		while target:exists() and (not target.Lootable) and (os.clock() - lootStart < .2) do
			yrest(20)
			target:updateLootable()
		end

		if target.Lootable then
			looten();
			stateman:popState(self:toString()..": complete");
			__botState.allowTargetting = self.oldAllowTargetting;
			if( self.TargetPtr ~= 0 ) then
				player:clearTarget();
			end
			return;
		else
			if( settings.profile.options.DEBUG_LOOT) then
				cprintf(cli.yellow, "[DEBUG] don't loot reason: target not lootable.\n");
				stateman:popState(self:toString()..": don't loot reason: target not lootable");
				__botState.allowTargetting = self.oldAllowTargetting;
				return;
			end;
		end;

		-- check for loot problems to give a noob mesassage
		player:updateTargetPtr()
		target = CPawn.new(player.TargetPtr)
		if target:exists() then
			target:updateLootable()
			if target.Lootable == true then	-- death mob disapeared?
				cprintf(cli.green, language[100]); -- We didn't move to the loot!?

				-- second loot try?
				if( type(settings.profile.options.LOOT_AGAIN) == "number" and settings.profile.options.LOOT_AGAIN > 0 ) then
					yrest(settings.profile.options.LOOT_AGAIN);
					looten();	-- try it again
					stateman:popState(self:toString()..": complete");
					__botState.allowTargetting = self.oldAllowTargetting;
					if( self.TargetPtr ~= 0 ) then
						player:clearTarget();
					end
					return;
				end
			end;
		end

		-- rnd pause from 2-6 sec after loot to look more human
		if( settings.profile.options.LOOT_PAUSE_AFTER > 0 ) then
			player:restrnd( settings.profile.options.LOOT_PAUSE_AFTER,2,6);
		end;

		-- Close the booty bag.
		RoMScript("BootyFrame:Hide()");
	end -- 'end' ends the 'if' statement

	local function sigilNameMatch(_name)
		if settings.profile.options.SIGILS_IGNORE_LIST == nil then
			return true -- collect all sigils
		end

		for sigil in string.gmatch(";"..settings.profile.options.SIGILS_IGNORE_LIST,"[,;]([^,;]*)") do
			if string.match(sigil,"^'.*'$") then sigil = string.match(sigil,"^'(.*)'$") end
			if sigil == _name then
				return false -- ignore the sigil
			end
		end

		return true -- not in ignore list. Don't ignore.
	end

	local function getNearestSigil()
		local nearestSigil = nil;
		local obj = nil;
		local objectList = __View:getObjectList();
		player:updateXYZ()

		for i = 0,objectList:size() do
			obj = objectList:getObject(i);

			if( obj ~= nil ) and ( obj.Type == PT_SIGIL ) and sigilNameMatch(obj.Name) then

				local dist = distance(player.X, player.Z, obj.X, obj.Z);

				if( nearestSigil == nil and dist < settings.profile.options.LOOT_DISTANCE ) then
					nearestSigil = obj;
				else

					if( dist < settings.profile.options.LOOT_DISTANCE and
						dist < distance(player.X, player.Z, nearestSigil.X, nearestSigil.Z) ) then
						-- New nearest sigil found
						nearestSigil = obj;
					end
				end
			end
		end

		return nearestSigil;
	end

	if settings.profile.options.LOOT_SIGILS == true or (settings.profile.options.LOOT == true and settings.profile.options.LOOT_SIGILS == nil) then
		-- Pick up all nearby sigils
		player:clearTarget();
		local sigil = getNearestSigil();
		if( sigil ) then
			player:updateXYZ()
			local dist = distance(player.X, player.Z, player.Y, sigil.X, sigil.Z, sigil.Y);
			local angle = math.atan2(sigil.Z - player.Z, sigil.X - player.X);
			local yangle = math.atan2(sigil.Y - player.Y, ((sigil.X - player.X)^2 + (sigil.Z - player.Z)^2)^.5 );
			local nY = player.Y + math.sin(yangle) * (dist + 15);
			local hypotenuse = (1 - math.sin(yangle)^2)^.5
			local nX = player.X + math.cos(angle) * (dist + 15) * hypotenuse;
			local nZ = player.Z + math.sin(angle) * (dist + 15) * hypotenuse;
			printf("Picking up sigil \"%s\"\n",sigil.Name)

			stateman:pushEvent("Move", "Loot-Sigil", { ["allowTargetAquisition"]=false; ["waypoint"]=CWaypoint(nX, nZ, nY);  ["range"] = 10; ["ignoreCombatEvent"]=self.ignoreCombatEvent} );
		end
	end
	
	if( self.TargetPtr ~= 0 ) then
		player:clearTarget();
	end	
	stateman:popState(self:toString()..": complete");
	__botState.allowTargetting = self.oldAllowTargetting;


end

function LootState:handleEvent(event)
	if (event == "Combat") and (self.ignoreCombatEvent == true) then
		return true;
	elseif (event == "Idle") then
		return true;
	end
end

function LootState:clearState()
end;


table.insert(events,{name = "Loot", func = LootState})
