--[[
	Child of State class.
]]

include("../state.lua");

MonitorChatsState = class(State);

function MonitorChatsState:constructor(args)
	self.name = "MonitorChats";
	self.mustBeDoneBy = args.mustBeDoneBy or -1;
end

function MonitorChatsState:update()
	logger:log(LL_DEBUG_STATES,self:toString()..":update()");

	if (self.mustBeDoneBy > -1) then
		local timeRemaining = self.mustBeDoneBy - os.clock();
		if (timeRemaining < 1) then
			stateman:popState(self:toString().." not enough time to read messages");
		end;
	end;
	
	local botState = __botState;
		
	if (botState.gameMonitors) then
		local monitors = botState.gameMonitors;
		
		if (monitors.active) then
			local clk = os.clock();
			if (((clk - 1) > botState.lastChatsSent)) then
				if (settings.profile.options.DEBUG_STATES == true) then
					printf("checking event monitors\n");
				end;
				
				local startTime = os.clock();
				--printf("must be done by: %s\n", tostring(self.mustBeDoneBy));
				if (monitors:checkForMessages(self.mustBeDoneBy) > 0) then
					local count = 0;
					while (not monitors.Q:isEmpty()) do
						local msg=monitors.Q:pop();
						printf("prepping: ["..tostring(msg.channel).."] "..msg.message.."\n");
						if (botState.sock) then
							botState.sock:send(__CP:CreateChatMessage(msg.channel, msg.name, msg.message, msg.time));
						end;
						count = count + 1;
					end;
					local timeTaken = os.clock()-startTime;
					printf("I checked the chats. It took "..tostring(timeTaken).."ms to read "..tostring(count).." chats. "..tostring(timeTaken/count).."ms per message\n");
				end;
				botState.lastChatsSent = clk;
			end;			
		end;
	end;

	stateman:popState(self:toString().." done reading messages");
end

function MonitorChatsState:handleEvent(event)
	if (event == "Idle") then
		return true;
	end;
	return false;
end

table.insert(events,{name = "MonitorChats", func = MonitorChatsState})