--[[
	Encapsulates the logic for moving the player to a waypoint
]]

include("../state.lua");

MOVE_FAILED = 0;
MOVE_SUCCEEDED = 1;

MoveState = class(State);

function MoveState:constructor(args)
	self.name = "MoveState";

-- parameters

	self.allowTargetAquisition = args.allowTargetAquisition or __botState.allowTargetting;
	self.oldAllowTargetting = __botState.allowTargetting
	self.ignoreCombatEvent = args.ignoreCombatEvent or false;
	self.waypoint = args.waypoint or {};
	self.range = args.range or 10;
	self.dontStop = args.dontStopAtEnd or false;

	self:clearState();

	if (self.waypoint == nil) then
		logger:log(LL_ERROR, 'Invalid waypont specified "' .. tostring(self.name)..'"');
	else
		self.name = sprintf("%s (%s)", self.name, tostring(self.waypoint));
	end;
end

function MoveState:update()
	logger:log(LL_DEBUG_STATES,self:toString()..":update()");
	__botState.allowTargetting = self.allowTargetAquisition;

	if (self.allowTargetAquisition == true) then
		--printf(self:toString().." searching for enemy\n");
		local t = player:findEnemy(false,nil,evalTargetDefault);
		
		if (t ~= nil) then
			player:target(t);
			stateman:popState(self:toString().." found target");
			return;
		end;
		
	end;	
	
	self.waypoint:updateXYZ();
	local wp = self.waypoint;
	if not wp then
		logger:log(LL_ERROR,"Error in waypoints '"..tostring(name).."'. Waypoint was not specified. Please check the waypoints or the waypoint file");
		--remove this waypoint
		stateman:popState(self:toString());
		__botState.allowTargetting = self.oldAllowTargetting;
		return;
	end


	local function passed_point(lastpos, point)
		point.X = tonumber(point.X)
		point.Z = tonumber(point.Z)

		local posbuffer = 5

		local passed = true
		if lastpos.X < point.X and player.X < point.X - posbuffer then
			return false
		end
		if lastpos.X > point.X and player.X > point.X + posbuffer then
			return false
		end
		if lastpos.Z < point.Z and player.Z < point.Z - posbuffer then
			return false
		end
		if lastpos.Z > point.Z and player.Z > point.Z + posbuffer then
			return false
		end

		return true
	end



	-- ******************************* Check direction *******************************
	
	--make sure we are facing the right direction
	player:updateXYZ();
	local angle = math.atan2(wp.Z - player.Z, wp.X - player.X);
	local yangle = 0
	if wp.Y ~= nil then
		yangle = math.atan2(wp.Y - player.Y, ((wp.X - player.X)^2 + (wp.Z - player.Z)^2)^.5 );
	end
	--player:updateDirection()
	local angleDif = angleDifference(angle, player.Direction);

	if (self.isTurning) then
		--continue to perform turning
		if( os.difftime(os.time(), self.turning.rotateStartTime) > 3.0 ) then
			-- Sometimes both left and right rotate get stuck down.
			-- Press them both to make sure they are fully released.
			keyboardPress(settings.hotkeys.ROTATE_LEFT.key);
			keyboardPress(settings.hotkeys.ROTATE_RIGHT.key);

			-- we seem to have been distracted, take a step back.
			stateman:pushEvent("HoldKey", "Move", {['key']=settings.hotkeys.MOVE_BACKWARD.key; ['duration']=500; ["ignoreCombatEvent"]=self.ignoreCombatEvent});
			return;
		end

		if ( angleDif > math.rad(65) ) then
			return; --carry on turning until we are facing the right direction
		else
			keyboardRelease( settings.hotkeys.ROTATE_LEFT.key );
			keyboardRelease( settings.hotkeys.ROTATE_RIGHT.key );
		end;
	end;


	-- QUICK_TURN only
	if( settings.profile.options.QUICK_TURN == true ) then
		player:faceDirection(angle, yangle);
		camera:setRotation(angle);
		angleDif = angleDifference(angle, player.Direction);
	else
		player:faceDirection(player.Direction, yangle); -- change only 'Y' angle with 'faceDirection'.
	end

	if ( angleDif > math.rad(65) ) then
		if( angleDifference(angle, player.Direction + 0.01) < angleDif ) then
			-- rotate left
			keyboardRelease( settings.hotkeys.ROTATE_RIGHT.key );
			keyboardHold( settings.hotkeys.ROTATE_LEFT.key );
		else
			-- rotate right
			keyboardRelease( settings.hotkeys.ROTATE_LEFT.key );
			keyboardHold( settings.hotkeys.ROTATE_RIGHT.key );
		end

		self.isTurning = true;
		self.turning = {};
		self.turning.rotateStartTime = os.time();
		self.turning.turningDir = -1;
		return;
	end;

	self.isTurning = false;
	self.turning = {};




	-- ******************************* Check moving *******************************

	--printf("Move: facing right direction, moving closer(I hope)\n");
	-- Direction ok, start moving forward
	local dist = distance(player.X, player.Z, player.Y, wp.X, wp.Z, wp.Y);
	local lastDist = self.moving.lastDistance or dist;
	local lastpos = self.moving.lastPos or {X=player.X, Z=player.Z, Y=player.Y}

	player.LastDistImprove = os.time();	-- global, because we reset it when we apply buffs/potions etc otherwise this might trigger the stuck state

	--dist = distance(player.X, player.Z, player.Y, wp.X, wp.Z, wp.Y);

	-- We're still making progress
	if( dist < lastDist ) then
		player.LastDistImprove = os.time();

		self.moving.lastDistance = dist;
		self.moving.lastPos = {X=player.X, Z=player.Z, Y=player.Y}
		lastPos = self.moving.lastPos;
		
	elseif(  dist > lastDist + 40 ) then
		--might need to drop off the stack and report failure to the caller
		self.result = MOVE_FAILED;
		self.failureReason = "Moved further away from our destination!!!"; --could be a lag pullback or overran or combat??
		printf("Move Failed: "..self.failureReason.."\n");
		stateman:popState(self:toString());
		__botState.allowTargetting = self.oldAllowTargetting;
		return;
	end;

	if( os.difftime(os.time(), player.LastDistImprove) > 3 ) then
		-- We haven't improved for 3 seconds, assume stuck
		self.result = MOVE_FAILED;
		self.failureReason = WF_STUCK;
		printf("Move Failed: "..self.failureReason.."\n");
		stateman:popState(self:toString());
		__botState.allowTargetting = self.oldAllowTargetting;
		return;
	end

	-- Check if within range if range specified
	if self.range and self.range > dist then
		self.result = MOVE_SUCCEEDED;
		printf("Move succeeded:"..tostring(dist).." < "..tostring(self.range).."\n");
		stateman:popState(self:toString());
		__botState.allowTargetting = self.oldAllowTargetting;
		return;
	end

	-- Check if past waypoint
	if passed_point(lastpos, wp) then
		self.result = MOVE_SUCCEEDED;
		--printf("Move succeeded\n");
		stateman:popState(self:toString());
		__botState.allowTargetting = self.oldAllowTargetting;
		return;
	end
	
	local distToGo = dist;
	player:updateActualSpeed();
	local spd = player.ActualSpeed or 50;
	if (spd < 1) then spd = 50; end;
	local wait = (distToGo / spd) * 1000;
	
	if (wait > 5000) then wait = 5000; end;
	
	if (wait < 50) then 
		wait = 50; 
	else
		wait = wait * 0.8; --I find we overrun too often by trying for the 'perfect' solution. - try this out later if necessary
	end;
	
	--printf("Holding move forward: "..distToGo.." vs "..spd.." wait: "..wait.."\n");
	stateman:pushEvent("HoldKey", "Move", {['key']=settings.hotkeys.MOVE_FORWARD.key; ['duration']=wait; ["ignoreCombatEvent"]=self.ignoreCombatEvent});
	
end

function MoveState:handleEvent(event)
	if (event == "Combat") and (self.ignoreCombatEvent == true) then
		return true;
	elseif (event=="Idle") then
		return true;
	end
end

function MoveState:clearState()
	self.moving = {};
	self.isTurning = false;
	self.turning = {};
	self.isHoldingKey = false;
	self.holding = {};
end;


table.insert(events,{name = "Move", func = MoveState})
