--[[
	Child of State class.
]]

include("../state.lua");

MoveInRange = class(State);

function MoveInRange:constructor(args)
	self.name = "MoveInRange";
	self.range = args.range;
end

local function faceTarget()

	if player.Cast_to_target == 0 or
		player.LastSkill.CastTime == 0 or
		player.LastSkill.ClickToCast ~= true then

		player:updateTargetPtr()
		local target = CPawn.new(player.TargetPtr)
		target:updateType()
		if not target:exists() or target.Type ~= PT_MONSTER then return end

		target:updateXYZ()
		if( settings.profile.options.QUICK_TURN ) then
			local angle = math.atan2(target.Z - player.Z, target.X - player.X);
			local yangle = math.atan2(target.Y - player.Y, ((target.X - player.X)^2 + (target.Z - player.Z)^2)^.5 );
			player:faceDirection(angle, yangle);

			camera:setRotation(angle);
		elseif( settings.options.ENABLE_FIGHT_SLOW_TURN ) then
			-- Make sure we're facing the enemy
			local angle = math.atan2(target.Z - player.Z, target.X - player.X);
			local yangle = math.atan2(target.Y - player.Y, ((target.X - player.X)^2 + (target.Z - player.Z)^2)^.5 );
			player:faceDirection(player.Direction, yangle); -- change only 'Y' angle with 'faceDirection'.
			local angleDif = angleDifference(angle, player.Direction);
			local correctingAngle = false;
			local startTime = os.time();

			while( angleDif > math.rad(15) ) do
				if( player.HP < 1 or player.Alive == false ) then
					player.Fighting = false;
					return;
				end;

				if( os.difftime(os.time(), startTime) > 5 ) then
					printf(language[26]);
					break;
				end;

				correctingAngle = true;
				if( angleDifference(angle, player.Direction + 0.01) < angleDif ) then
					-- rotate left
					keyboardRelease( settings.hotkeys.ROTATE_RIGHT.key );
					keyboardHold( settings.hotkeys.ROTATE_LEFT.key );
				else
					-- rotate right
					keyboardRelease( settings.hotkeys.ROTATE_LEFT.key );
					keyboardHold( settings.hotkeys.ROTATE_RIGHT.key );
				end

				yrest(100);
				player:updateDirection();
				target:updateXYZ();
				angle = math.atan2(target.Z - player.Z, target.X - player.X);
				angleDif = angleDifference(angle, player.Direction);
			end

			if( correctingAngle ) then
				keyboardRelease( settings.hotkeys.ROTATE_LEFT.key );
				keyboardRelease( settings.hotkeys.ROTATE_RIGHT.key );
			end

		end
	end
end

function MoveInRange:update()
	logger:log(LL_DEBUG_STATES,"Coming to MoveInRange:update()");
	--printf("Cmbat: update: "..tostring(self.currentState).."\n");
	if (not player:haveTarget()) then
		--check for aggro??? - remember to ignore mobs on the ignore list as we may have just ended a long running fight and now need to stop targetting said mob
		--stateman:pushEvent("Loot", "Combat");
		stateman:popState();
		return;
	end;

	if (self.currentState > CS_INITIATING) then
		self.fightStartTime = getGameTime();
	end;

	player:updateHP();
	player:updateAlive();
	player:updateBattling();

	player:checkPotions();

	player:updateMounted()
	if player.Mounted then
		player:dismount()
	end

	local target = CPawn.new(player.TargetPtr);

	--End the fight if it has gone over time
	if (self.currentState > CS_INITIATING) then
		if getGameTime() - self.fightStartTime > settings.profile.options.MAX_FIGHT_TIME then
			target:updateLastDamage()
			player:updateLastHitTime()
			if target.LastDamage == 0 or (getGameTime() - player.LastHitTime) > settings.profile.options.MAX_FIGHT_TIME then
				printf(language[83]);			-- Taking too long to damage target
				player:addToMobIgnoreList(target.Address)
				player:clearTarget();

				player:updateBattling()
				if( player.Battling ) then
					stateman:pushEvent("HoldKey", "Combat", {['key']=settings.hotkeys.MOVE_BACKWARD.key; ['duration']=1000});
				end

				player.break_fight = true;
				return;
			end
		end
	end;

	--Check for Elite fights

	if (not self.AutoEliteDone) and (target.MaxHP > (player.MaxHP * settings.profile.options.AUTO_ELITE_FACTOR)) then
		-- check if preCodeOnElite event is used in profile
		if( type(settings.profile.events.preCodeOnElite) == "function" ) then
			releaseKeys();
			_arg1 = target
			local status,err = pcall(settings.profile.events.preCodeOnElite);
			if( status == false ) then
				local msg = sprintf(language[188], err);
				error(msg);
			end
		end
		self.AutoEliteDone = true;
	end

	if (self.currentState < CS_FIGHTING) then
		if player.Class1 == CLASS_WARDEN then -- if warden let pet start fight.
			petupdate()
			if pet.Name == GetIdName(102297) or
			pet.Name == GetIdName(102324) or
			pet.Name == GetIdName(102803)
			then
				petstartcombat()
			end
		end
	end;

	if ((player.Battling == true) and (settings.profile.options.COMBAT_TYPE == "ranged")) then
		self.currentState = CS_FIGHTING;
	end;

	if (self.currentState < CS_PULLING) then
	
		pullIsRanged = false;

		--perform the initial attack
		if((settings.profile.options.COMBAT_TYPE == "melee")  and (settings.profile.options.COMBAT_RANGED_PULL == true) and (player.Battling ~= true) ) then
			pullIsRanged = true;
			cprintf(cli.green, language[96]);	-- we start with ranged pulling
		elseif ((settings.profile.options.COMBAT_TYPE == "melee") and (settings.profile.options.COMBAT_RANGED_PULL == false)) then
			--printf("Am MELEE and not RANGE_PULL\n");
			self.currentState = CS_FIGHTING;
			Attack();
			return;
		end

		self.pullStartDistance = distance(player.X, player.Z, target.X, target.Z);
		self.pullStartTime = os.time();
		self.currentState = CS_PULLING;
	elseif (self.currentState == CS_PULLING) then
		local dist = distance(player.X, player.Z, target.X, target.Z);

		if( dist <= settings.options.MELEE_DISTANCE ) then
			cprintf(cli.green, language[97]); -- Ranged pulling finished, mob in melee distance
			self.pullIsRanged = false;
		elseif( os.difftime(os.time(), self.pullStartTime) > 3 and self.pullStartTime ~= 0 ) then
			cprintf(cli.green, language[98]); -- Ranged pulling after 3 sec wait finished
			self.pullIsRanged = false;
		elseif( dist >=  self.pullStartDistance-45 and	-- mob not really coming closer
		  os.difftime(os.time(), player.aggro_start_time) > 1  and player.aggro_start_time ~= 0 ) then
			cprintf(cli.green, language[99]); -- Ranged pulling finished. Mob not really moving
			self.pullIsRanged = false;
		end;

		if ((self.pullIsRanged == false) and (settings.profile.options.COMBAT_TYPE == "melee")) then
			Attack();
			self.currentState = CS_FIGHTING;
		elseif (settings.profile.options.COMBAT_TYPE == "ranged") then
			self.currentState = CS_FIGHTING;
		end;
	elseif (self.currentState == CS_FIGHTING) then
		target:updateXYZ();
		player:updateXYZ();
		local dist = distance(player.X, player.Z, target.X, target.Z);

		player:updateCasting()
		if( dist < 5.0 and not player.Casting ) then
			printf(language[24]);
			stateman:pushEvent("HoldKey", "Combat", { ["key"]=settings.hotkeys.MOVE_BACKWARD.key; ["duration"]=200} );
			return;
		end

		-- Move closer to the target if needed
		local suggestedRange = settings.options.MELEE_DISTANCE;
		if( suggestedRange == nil ) then suggestedRange = 45; end;
		if( settings.profile.options.COMBAT_TYPE == "ranged") then
			if( settings.profile.options.COMBAT_DISTANCE ~= nil ) then
				suggestedRange = settings.profile.options.COMBAT_DISTANCE;
			else
				suggestedRange = 155;
			end
		end

		if( dist > suggestedRange and not player.Casting ) then

			if (self.moveCloserCounter == nil) then self.moveCloserCounter = 0; end;
			self.moveCloserCounter = self.moveCloserCounter + 1;

			if (self.moveCloserCounter > 3) then
				cprintf(cli.green, language[84]);	-- To much tries to come closer
				stateman:popState("Combat");
				return;
			end;

			printf(language[25], suggestedRange, dist);

			stateman:pushEvent("MoveInRange", "Combat", { ["range"] = suggestedRange} );
			return;
		end;

		faceTarget();

		if (player:checkPotions() or player:checkSkills()) then
			player.LastDistImprove = os.time();
		end;

		if (player.break_fight) then
			cprintf(cli.green, language[19]);	-- Target lost
			stateman:popState("Combat");
		end;
	end

end

-- Handle events
function MoveInRange:handleEvent(event)
	if (event == "Combat") then
		--check to make sure whoever is attacking us isn't the same as our existing target - i.e. we are running away and have run into somoene else!
		if (player.TargetPtr <1) then return true; end;
		if (self.currentPlayerTarget == player.TargetPtr) then return true; end;
		if (self.currentState < CS_FIGHTING) then return false; end;
		return true;
	end;
end

table.insert(events,{name = "MoveInRange", func = MoveInRange})
