--[[
	Encapsulates the logic for moving the player to a waypoint
]]

include("../state.lua");

WLES_LOADING = 0;
WLES_STARTING = WLES_LOADING + 1;
WLES_HEADING_TO_WP = WLES_STARTING + 1;

WaypointListExecuterState = class(State);

function WaypointListExecuterState:constructor(args)
	self.name = "WaypointListExecuterState";

-- parameters
	self.currentState = WLES_LOADING;
	self.waypointListName = args.waypointListName or "<un-named waypoint list>";
	self.waypointList = args.waypointList or {};
	self.lastPlayerPos = { X=player.X, Z=player.Z, Y=player.Y };
	self.oldState = {};
	self.oldState.MAX_TARGET_DIST = settings.profile.options.MAX_TARGET_DIST;
	self.oldState.mobs = settings.profile.mobs;
	self.oldState.friends = settings.profile.friends;
	
	self.expectingMoved = false;
	self.stuckCount = 0;
	self.waypointList.executor = self;
	__WPL = self.waypointList;
end

function WaypointListExecuterState:update()
	logger:log(LL_DEBUG_STATES,self:toString()..":update()");
	
	if (self.currentState == WLES_LOADING) then
		--run the onLoad code
		printf("WLES_LOADING\n");
		if (self.waypointList.onLoadEvent ~= nil) then
			self.waypointList.onLoadEvent(self);
		end;
		
		if ((self.waypointList.Mode == "waypoints") and (#self.waypointList.Waypoints == 0)) then
			printf(language[114].."\n");
			stateman:popState(self:toString()..language[114]);
			return;
		end;
		
		local wp = self.waypointList.Waypoints[self.waypointList.CurrentWaypoint];
		if (wp) then
			printf("Starting at waypoint: %s\n", wp:toString());
		else
			printf("No starting waypoint found!!!! Unloading.\n");
			stateman:popState(self:toString());
			return;
		end;
		self.lastWaypointIndex = self.waypointList.CurrentWaypoint;
		self.lastDistanceToWaypoint = distance(player.X, player.Z, player.Y, wp.X, wp.Z, wp.Y);
		self.currentState = WLES_STARTING;
	elseif (self.currentState == WLES_STARTING) then
		printf("WLES_STARTING\n");
		--locate the starting waypoint
		local wp = self.waypointList.Waypoints[self.waypointList.CurrentWaypoint];
		self.lastPlayerPos = { X=player.X, Z=player.Z, Y=player.Y };
		stateman:pushEvent("Move", self:toString(), { ignoreCombatEvent=wp.Type ~= WPT_TRAVEL, ["waypoint"]=wp, range=10 });
--		self.expectingMoved = true;
		self.currentState = WLES_HEADING_TO_WP;
		
	elseif (self.currentState == WLES_HEADING_TO_WP) then
		printf("WLES_HEADING_TO_WP\n");
		player:updateMounted()
		if not player.Mounted then
			player:checkPotions();
			--player:checkSkills( ONLY_FRIENDLY );	-- only cast hot spells to ourselfe
		end

		local wp = self.waypointList.Waypoints[self.waypointList.CurrentWaypoint];
		--it is possible that we have been interrupted by combat etc so maybe we should check to see if we are at a new waypoint and determine if we missed any code??
--		if (self.expectingMoved) then
			local distToWP = distance(player.X, player.Z, player.Y, wp.X, wp.Z, wp.Y);
			local dist = distance(player.X, player.Z, player.Y, self.lastPlayerPos.X, self.lastPlayerPos.Z, self.lastPlayerPos.Y);
			local deviation = settings.profile.options.WAYPOINT_DEVIATION or 5;

			if (distToWP > deviation) then
				if (dist < 1) then
					self.stuckCount = self.stuckCount + 1;
					if (self.stuckCount < 3) then
						self.lastPlayerPos = { X=player.X, Z=player.Z, Y=player.Y };
						stateman:pushEvent("Move", self:toString(), { ignoreCombatEvent=wp.Type ~= WPT_TRAVEL, ["waypoint"]=wp, range=deviation });
	--					self.expectingMoved = true;
						return;
					end;
					
					printf("WERE STUCCCCKKKKKKK");
					stateman:popState(self:toString().." we were STUCK");
				end;
				self.stuckCount = 0;
			end;
			self.stuckCount = 0;

			printf(self.waypointList.CurrentWaypoint..": %s - complete.\n", wp:toString());
			--we're making progress from where we were
			checkEggPets();
			
			if( wp.Action and type(wp.Action) == "string" and string.find(wp.Action,"%a") ) then
				local actionchunk = loadstring(wp.Action);
				assert( actionchunk,  sprintf(language[150], self.waypointList.CurrentWaypoint) );
				actionchunk();
			end;

			self.waypointList:advance();
			
			self.lastPlayerPos = { X=player.X, Z=player.Z, Y=player.Y };
			stateman:pushEvent("Move", self:toString(), { ignoreCombatEvent=wp.Type ~= WPT_TRAVEL, ["waypoint"]=wp, range=deviation });
--			self.expectingMoved = true;
--		end;
		
		
	end;
end

function WaypointListExecuterState:handleEvent(event)
	if (event == "Combat") then
		local canFight = true;
		canFight = canFight and ((self.waypointList) and ((self.waypointList.Type == NORMAL) or (self.waypointList.Type == RUN)));
		if (canFight and self.waypointList.CurrentWaypoint) then
			local wp = self.waypointList.Waypoints[self.waypointList.CurrentWaypoint];
			canFight = canFight and (((wp.Type == NORMAL) or (wp.Type == RUN)) and (not self.waypointList.Type == WPT_TRAVEL));
		end;
		
		return canFight;
	elseif (event=="Idle") then
		return true;
	end
end

function WaypointListExecuterState:clearState()
end;


table.insert(events,{name = "WaypointListExecuter", func = WaypointListExecuterState})
